"""Entrypoint."""

import asyncio
import logging
import signal
import sys
from typing import Sequence

from configargparse import ArgumentParser

from ..config.error import ArgsParseError

try:
    import uvloop
except ImportError:
    uvloop = None

from ..config import argparse as arg
from ..config.default_context import DefaultContextBuilder
from ..config.util import common_config
from ..core.conductor import Conductor
from . import PROG

LOGGER = logging.getLogger(__name__)


async def start_app(conductor: Conductor):
    """Start up the application."""
    await conductor.setup()
    await conductor.start()


async def shutdown_app(conductor: Conductor):
    """Shut down the application."""
    LOGGER.info("Shutting down")
    await conductor.stop()

    # Cancel remaining tasks
    tasks = [t for t in asyncio.all_tasks() if t is not asyncio.current_task()]
    for task in tasks:
        task.cancel()
    await asyncio.gather(*tasks, return_exceptions=True)


def init_argument_parser(parser: ArgumentParser):
    """Initialize an argument parser with the module's arguments."""
    return arg.load_argument_groups(parser, *arg.group.get_registered(arg.CAT_START))


async def run_app(argv: Sequence[str] = None):
    """Main async runner for the app."""
    parser = arg.create_argument_parser(prog=PROG)
    parser.prog += " start"
    get_settings = init_argument_parser(parser)
    args = parser.parse_args(argv)
    settings = get_settings(args)
    common_config(settings)

    # Set ledger to read-only if explicitly specified
    settings["ledger.read_only"] = settings.get("read_only_ledger", False)

    if uvloop:
        uvloop.install()
        LOGGER.info("uvloop installed")

    context_builder = DefaultContextBuilder(settings)
    conductor = Conductor(context_builder)

    loop = asyncio.get_running_loop()
    shutdown_event = asyncio.Event()

    def handle_signal():
        LOGGER.info("Received stop signal")
        shutdown_event.set()

    loop.add_signal_handler(signal.SIGTERM, handle_signal)
    loop.add_signal_handler(signal.SIGINT, handle_signal)

    try:
        await start_app(conductor)
        await shutdown_event.wait()
    finally:
        await shutdown_app(conductor)


def execute(argv: Sequence[str] = None):
    """Entrypoint."""
    try:
        asyncio.run(run_app(argv))
    except ArgsParseError as e:
        LOGGER.error("Argument parsing error: %s", e)
        raise e
    except KeyboardInterrupt:
        LOGGER.info("Interrupted by user")
    except Exception:
        LOGGER.exception("Unexpected exception during execution")
        sys.exit(1)


def main():
    """Execute the main line."""
    execute()


if __name__ == "__main__":
    main()



from unittest import IsolatedAsyncioTestCase

from ....utils.testing import create_test_profile
from .. import issuer_cred_rev_record as test_module
from ..issuer_cred_rev_record import IssuerCredRevRecord

TEST_DID = "55GkHamhTU1ZbTbV2ab9DE"
CRED_DEF_ID = f"{TEST_DID}:3:CL:1234:default"
REV_REG_ID = f"{TEST_DID}:4:{CRED_DEF_ID}:CL_ACCUM:0"


class TestIssuerCredRevRecord(IsolatedAsyncioTestCase):
    async def asyncSetUp(self):
        self.profile = await create_test_profile()

    async def test_serde(self):
        rec = IssuerCredRevRecord(
            record_id=test_module.UUID4_EXAMPLE,
            state=IssuerCredRevRecord.STATE_ISSUED,
            cred_ex_id=test_module.UUID4_EXAMPLE,
            rev_reg_id=REV_REG_ID,
            cred_rev_id="1",
        )
        ser = rec.serialize()
        assert ser["record_id"] == rec.record_id
        assert ser["cred_def_id"] == CRED_DEF_ID
        assert rec.cred_def_id == CRED_DEF_ID

        assert rec == IssuerCredRevRecord.deserialize(ser)

    async def test_rec_ops(self):
        recs = [
            IssuerCredRevRecord(
                state=IssuerCredRevRecord.STATE_ISSUED,
                cred_ex_id=test_module.UUID4_EXAMPLE,
                rev_reg_id=REV_REG_ID,
                cred_rev_id=str(i + 1),
            )
            for i in range(2)
        ]

        async with self.profile.session() as session:
            # Save and update first record
            await recs[0].set_state(
                session,
                IssuerCredRevRecord.STATE_REVOKED,
            )

            # Confirm the two records have different revocation IDs
            assert recs[0].cred_rev_id != recs[1].cred_rev_id

            # Helper to simplify record comparison
            def strip(record):
                return {
                    "cred_ex_id": record.cred_ex_id,
                    "cred_rev_id": record.cred_rev_id,
                    "rev_reg_id": record.rev_reg_id,
                    "cred_def_id": record.cred_def_id,
                    "state": record.state,
                }

            # Query and compare based on stripped fields
            result = await IssuerCredRevRecord.query_by_ids(session)
            assert strip(result[0]) == strip(recs[0])

            result = await IssuerCredRevRecord.retrieve_by_cred_ex_id(
                session, test_module.UUID4_EXAMPLE
            )
            assert strip(result) == strip(recs[0])

            result = await IssuerCredRevRecord.query_by_ids(
                session, cred_def_id=CRED_DEF_ID
            )
            assert strip(result[0]) == strip(recs[0])

            result = await IssuerCredRevRecord.query_by_ids(
                session, rev_reg_id=REV_REG_ID
            )
            assert strip(result[0]) == strip(recs[0])

            result = await IssuerCredRevRecord.query_by_ids(
                session, cred_def_id=CRED_DEF_ID, rev_reg_id=REV_REG_ID
            )
            assert strip(result[0]) == strip(recs[0])

            result = await IssuerCredRevRecord.query_by_ids(
                session, state=IssuerCredRevRecord.STATE_REVOKED
            )
            assert strip(result[0]) == strip(recs[0])

            result = await IssuerCredRevRecord.query_by_ids(
                session, state=IssuerCredRevRecord.STATE_ISSUED
            )
            assert not result

            # Save second record
            await recs[1].set_state(
                session,
                IssuerCredRevRecord.STATE_REVOKED,
            )

            result = await IssuerCredRevRecord.retrieve_by_ids(
                session, rev_reg_id=REV_REG_ID, cred_rev_id="1"
            )
            assert [strip(r) for r in result] == [strip(recs[0])]

            result = await IssuerCredRevRecord.retrieve_by_ids(
                session, rev_reg_id=REV_REG_ID, cred_rev_id=["2"]
            )
            assert [strip(r) for r in result] == [strip(recs[1])]

            result = await IssuerCredRevRecord.retrieve_by_ids(
                session, rev_reg_id=REV_REG_ID, cred_rev_id=["1", "2"]
            )
            assert sorted(
                [strip(r) for r in result], key=lambda r: r["cred_rev_id"]
            ) == sorted([strip(r) for r in recs], key=lambda r: r["cred_rev_id"])

            result = await IssuerCredRevRecord.retrieve_by_ids(
                session, rev_reg_id=REV_REG_ID, cred_rev_id=["3"]
            )
            assert result == []



"""Classes to manage credential revocation."""

import logging
from collections.abc import Mapping, Sequence
from typing import Optional, Text, Tuple, Type

from ..anoncreds.revocation import AnonCredsRevocation
from ..core.error import BaseError
from ..core.profile import Profile
from ..protocols.issue_credential.v1_0.models.credential_exchange import (
    V10CredentialExchange,
)
from ..protocols.issue_credential.v2_0.models.cred_ex_record import V20CredExRecord
from ..protocols.revocation_notification.v1_0.models.rev_notification_record import (
    RevNotificationRecord,
)
from ..revocation.util import notify_pending_cleared_event
from ..storage.error import StorageNotFoundError
from .models.issuer_cred_rev_record import IssuerCredRevRecord


class RevocationManagerError(BaseError):
    """Revocation manager error."""


class RevocationManager:
    """Class for managing revocation operations."""

    def __init__(self, profile: Profile):
        """Initialize a RevocationManager.

        Args:
            profile: The profile instance for this revocation manager
        """
        self._profile = profile
        self._logger = logging.getLogger(__name__)

    async def revoke_credential_by_cred_ex_id(
        self,
        cred_ex_id: str,
        publish: bool = False,
        notify: bool = False,
        notify_version: Optional[str] = None,
        thread_id: Optional[str] = None,
        connection_id: Optional[str] = None,
        comment: Optional[str] = None,
        options: Optional[dict] = None,
    ):
        """Revoke a credential by its credential exchange identifier at issue.

        Optionally, publish the corresponding revocation registry delta to the ledger.

        Args:
            cred_ex_id (str): The credential exchange identifier.
            publish (bool, optional): Whether to publish the resulting revocation
                registry delta, along with any revocations pending against it.
                Defaults to False.
            notify (bool, optional): Whether to notify the issuer of the revocation.
                Defaults to False.
            notify_version (str, optional): The version of the revocation notification to
                use. Defaults to None.
            thread_id (str, optional): The thread identifier for the revocation.
                Defaults to None.
            connection_id (str, optional): The connection identifier for the revocation.
                Defaults to None.
            comment (str, optional): A comment for the revocation. Defaults to None.
            options (dict, optional): Additional options for the revocation. Defaults to
                None.

        Raises:
            RevocationManagerError: If no issuer credential revocation record is found
                for the given credential exchange id.

        Returns:
            dict: The result of the credential revocation operation.

        """
        try:
            async with self._profile.session() as session:
                rec = await IssuerCredRevRecord.retrieve_by_cred_ex_id(
                    session,
                    cred_ex_id,
                )
        except StorageNotFoundError as err:
            raise RevocationManagerError(
                "No issuer credential revocation record found for "
                f"credential exchange id {cred_ex_id}"
            ) from err

        return await self.revoke_credential(
            rev_reg_id=rec.rev_reg_id,
            cred_rev_id=rec.cred_rev_id,
            publish=publish,
            notify=notify,
            notify_version=notify_version,
            thread_id=thread_id,
            connection_id=connection_id,
            comment=comment,
            options=options,
        )

    async def revoke_credential(
        self,
        rev_reg_id: str,
        cred_rev_id: str,
        publish: bool = False,
        notify: bool = False,
        notify_version: Optional[str] = None,
        thread_id: Optional[str] = None,
        connection_id: Optional[str] = None,
        comment: Optional[str] = None,
        options: Optional[dict] = None,
    ):
        """Revoke a credential.

        Optionally, publish the corresponding revocation registry delta to the ledger.

        Args:
            rev_reg_id (str): The revocation registry id.
            cred_rev_id (str): The credential revocation id.
            publish (bool, optional): Whether to publish the resulting revocation
                registry delta, along with any revocations pending against it.
                Defaults to False.
            notify (bool, optional): Whether to send a revocation notification.
                Defaults to False.
            notify_version (str, optional): The version of the revocation notification.
                Defaults to None.
            thread_id (str, optional): The thread id for the revocation notification.
                Defaults to None.
            connection_id (str, optional): The connection id for the revocation
                notification. Defaults to None.
            comment (str, optional): A comment for the revocation notification.
                Defaults to None.
            options (dict, optional): Additional options for revocation. Defaults to None.

        Raises:
            RevocationManagerError: If no revocation registry record is found for the
                given rev_reg_id.

        """
        revoc = AnonCredsRevocation(self._profile)
        rev_reg_def = await revoc.get_created_revocation_registry_definition(rev_reg_id)
        if not rev_reg_def:
            raise RevocationManagerError(
                f"No revocation registry record found for id: {rev_reg_id}"
            )

        if notify:
            thread_id = thread_id or f"indy::{rev_reg_id}::{cred_rev_id}"
            rev_notify_rec = RevNotificationRecord(
                rev_reg_id=rev_reg_id,
                cred_rev_id=cred_rev_id,
                thread_id=thread_id,
                connection_id=connection_id,
                comment=comment,
                version=notify_version,
            )
            async with self._profile.session() as session:
                await rev_notify_rec.save(session, reason="New revocation notification")
        if publish:
            await revoc.get_or_fetch_local_tails_path(rev_reg_def)
            result = await revoc.revoke_pending_credentials(
                rev_reg_id,
                additional_crids=[int(cred_rev_id)],
            )

            if result.curr and result.revoked:
                await self.set_cred_revoked_state(rev_reg_id, result.revoked)
                await revoc.update_revocation_list(
                    rev_reg_id,
                    result.prev,
                    result.curr,
                    result.revoked,
                    options=options,
                )

        else:
            await revoc.mark_pending_revocations(rev_reg_id, int(cred_rev_id))

    async def update_rev_reg_revoked_state(
        self,
        rev_reg_def_id: str,
        apply_ledger_update: bool,
        genesis_transactions: str,
    ) -> Tuple[dict, dict, dict]:
        """Request handler to fix ledger entry of credentials revoked against registry.

        This is an indy registry specific operation.

        Args:
            rev_reg_def_id: revocation registry definition id
            apply_ledger_update: whether to apply an update to the ledger
            genesis_transactions: genesis transactions for the ledger

        Returns:
            Number of credentials posted to ledger

        """
        revoc = AnonCredsRevocation(self._profile)
        rev_list = await revoc.get_created_revocation_list(rev_reg_def_id)
        if not rev_list:
            raise RevocationManagerError(
                f"No revocation list found for revocation registry id {rev_reg_def_id}"
            )

        raise RevocationManagerError(
            "Indy registry does not support revocation registry "
            f"identified by {rev_reg_def_id}"
        )

    async def publish_pending_revocations(
        self,
        rrid2crid: Optional[Mapping[Text, Sequence[Text]]] = None,
        options: Optional[dict] = None,
    ) -> Mapping[Text, Sequence[Text]]:
        """Publish pending revocations to the ledger.

        Args:
            rrid2crid: Mapping from revocation registry identifiers to all credential
                revocation identifiers within each to publish. Specify null/empty map
                for all revocation registries. Specify empty sequence per revocation
                registry identifier for all pending within the revocation registry;
                e.g.,

                {} - publish all pending revocations from all revocation registries

                {
                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:0": [],
                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:1": ["1", "2"]

                } - publish:
                    - all pending revocations from all revocation registry tagged 0
                    - pending ["1", "2"] from revocation registry tagged 1
                    - no pending revocations from any other revocation registries.
            options: Additional options for the revocation registry publish

        Returns: mapping from each revocation registry id to its cred rev ids published.
        """
        options = options or {}
        published_crids = {}
        revoc = AnonCredsRevocation(self._profile)

        rev_reg_def_ids = await revoc.get_revocation_lists_with_pending_revocations()
        for rrid in rev_reg_def_ids:
            if rrid2crid:
                if rrid not in rrid2crid:
                    continue
                limit_crids = [int(crid) for crid in rrid2crid[rrid]]
            else:
                limit_crids = None

            result = await revoc.revoke_pending_credentials(rrid, limit_crids=limit_crids)
            if result.curr and result.revoked:
                await self.set_cred_revoked_state(rrid, result.revoked)
                await revoc.update_revocation_list(
                    rrid, result.prev, result.curr, result.revoked, options
                )
                published_crids[rrid] = sorted(result.revoked)

        return published_crids

    async def clear_pending_revocations(
        self, purge: Mapping[Text, Sequence[Text]] = None
    ) -> Mapping[Text, Sequence[Text]]:
        """Clear pending revocation publications.

        Args:
            purge: Mapping from revocation registry identifiers to all credential
                revocation identifiers within each to clear. Specify null/empty map
                for all revocation registries. Specify empty sequence per revocation
                registry identifier for all pending within the revocation registry;
                e.g.,

                {} - clear all pending revocations from all revocation registries

                {
                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:0": [],
                    "R17v42T4pk...:4:R17v42T4pk...:3:CL:19:tag:CL_ACCUM:1": ["1", "2"]

                } - clear
                    - all pending revocations from all revocation registry tagged 0
                    - pending ["1", "2"] from revocation registry tagged 1
                    - no pending revocations from any other revocation registries.

        Returns:
            mapping from revocation registry id to its remaining
            cred rev ids still marked pending, omitting revocation registries
            with no remaining pending publications.

        """
        result = {}
        notify = []

        revoc = AnonCredsRevocation(self._profile)
        rrids = await revoc.get_revocation_lists_with_pending_revocations()
        async with self._profile.transaction() as txn:
            for rrid in rrids:
                await revoc.clear_pending_revocations(
                    txn,
                    rrid,
                    crid_mask=[int(crid) for crid in (purge or {}).get(rrid, ())],
                )
                remaining = await revoc.get_pending_revocations(rrid)
                if remaining:
                    result[rrid] = remaining
                notify.append(rrid)
            await txn.commit()

        for rrid in notify:
            await notify_pending_cleared_event(self._profile, rrid)

        return result

    async def set_cred_revoked_state(
        self, rev_reg_id: str, cred_rev_ids: Sequence[int]
    ) -> None:
        """Update credentials state to credential_revoked.

        Args:
            rev_reg_id: revocation registry ID
            cred_rev_ids: list of credential revocation IDs

        Returns:
            None

        """
        self._logger.debug(
            "Setting credential revoked state for %d credentials in rev_reg_id=%s",
            len(cred_rev_ids),
            rev_reg_id,
        )
        cred_rev_ids = [str(_id) for _id in cred_rev_ids]  # Method expects strings
        updated_cred_rev_ids = []  # Track updated to know if any were not found

        async with self._profile.transaction() as txn:
            # Retrieve all requested credential revocation records
            cred_rev_records = await IssuerCredRevRecord.retrieve_by_ids(
                txn, rev_reg_id, cred_rev_ids, for_update=True
            )

            # Update each record to indicate revoked
            for record in cred_rev_records:
                cred_rev_id = record.cred_rev_id
                self._logger.debug(
                    "Updating IssuerCredRevRecord for cred_rev_id=%s", cred_rev_id
                )
                record.state = IssuerCredRevRecord.STATE_REVOKED
                await record.save(txn, reason="revoke credential")
                self._logger.debug(
                    "Updated IssuerCredRevRecord state to REVOKED for cred_rev_id=%s",
                    cred_rev_id,
                )
                updated_cred_rev_ids.append(cred_rev_id)

            await txn.commit()

        # Check if any requested cred rev ids were not found
        missing_cred_rev_ids = [
            _id for _id in cred_rev_ids if _id not in updated_cred_rev_ids
        ]
        if missing_cred_rev_ids:
            self._logger.warning(
                "IssuerCredRevRecord not found for cred_rev_id=%s. Could not revoke.",
                missing_cred_rev_ids,
            )

        # Map cred_ex_version to the record type
        _cred_ex_version_map: dict[str, Type[V10CredentialExchange | V20CredExRecord]] = {
            IssuerCredRevRecord.VERSION_1: V10CredentialExchange,
            IssuerCredRevRecord.VERSION_2: V20CredExRecord,
        }

        # Update CredEx records for each credential revocation record
        for cred_rev_record in cred_rev_records:
            async with self._profile.transaction() as txn:
                cred_ex_id = cred_rev_record.cred_ex_id
                cred_ex_version = cred_rev_record.cred_ex_version
                known_record_type = _cred_ex_version_map.get(cred_ex_version)

                # If we know the record type, use it, otherwise try both V1 and V2
                record_type_to_use = (
                    [known_record_type]
                    if known_record_type
                    else [V10CredentialExchange, V20CredExRecord]
                )

                for record_type in record_type_to_use:
                    try:
                        cred_ex_record = await record_type.retrieve_by_id(
                            txn, cred_ex_id, for_update=True
                        )

                        # Update cred ex record state to indicate revoked
                        cred_ex_record.state = record_type.STATE_CREDENTIAL_REVOKED
                        await cred_ex_record.save(txn, reason="revoke credential")

                        self._logger.debug(
                            "Updated %s state to REVOKED for cred_ex_id=%s",
                            record_type.__name__,
                            cred_ex_id,
                        )
                        await txn.commit()
                        break  # Record found, no need to check other record type
                    except StorageNotFoundError:
                        # Credential Exchange records may have been deleted, which is fine
                        self._logger.debug(
                            "%s not found for cred_ex_id=%s / cred_rev_id=%s.%s",
                            record_type.__name__,
                            cred_ex_id,
                            cred_rev_record.cred_rev_id,
                            " Checking next record type."
                            if not known_record_type
                            else "",
                        )



import json
from unittest import IsolatedAsyncioTestCase

from aries_askar import AskarError, AskarErrorCode, Store

from acapy_agent.tests import mock

from ...core.error import ProfileDuplicateError, ProfileError, ProfileNotFoundError
from ..store import AskarOpenStore, AskarStoreConfig


class TestStoreConfig(IsolatedAsyncioTestCase):
    key_derivation_method = "Raw"
    key = "key"
    storage_type = "postgres"

    async def test_init_success(self):
        config = {
            "key_derivation_method": self.key_derivation_method,
            "key": self.key,
            "storage_type": self.storage_type,
        }

        askar_store = AskarStoreConfig(config)

        assert askar_store.key_derivation_method == self.key_derivation_method
        assert askar_store.key == self.key
        assert askar_store.storage_type == self.storage_type


class TestStoreOpen(IsolatedAsyncioTestCase):
    key_derivation_method = "Raw"
    key = "key"
    storage_type = "default"

    @mock.patch.object(Store, "open", autospec=True)
    async def test_open_store(self, mock_store_open):
        config = {
            "key_derivation_method": self.key_derivation_method,
            "key": self.key,
            "storage_type": self.storage_type,
        }

        store = await AskarStoreConfig(config).open_store()
        assert isinstance(store, AskarOpenStore)
        assert mock_store_open.called

    @mock.patch.object(Store, "open")
    async def test_open_store_fails(self, mock_store_open):
        config = {
            "key_derivation_method": self.key_derivation_method,
            "key": self.key,
            "storage_type": self.storage_type,
        }

        mock_store_open.side_effect = [
            AskarError(AskarErrorCode.NOT_FOUND, message="testing"),
            AskarError(AskarErrorCode.DUPLICATE, message="testing"),
            AskarError(AskarErrorCode.ENCRYPTION, message="testing"),
        ]

        with self.assertRaises(ProfileNotFoundError):
            await AskarStoreConfig(config).open_store()
        with self.assertRaises(ProfileDuplicateError):
            await AskarStoreConfig(config).open_store()
        with self.assertRaises(ProfileError):
            await AskarStoreConfig(config).open_store()

    @mock.patch.object(Store, "open")
    @mock.patch.object(Store, "rekey")
    async def test_open_store_fail_retry_with_rekey(self, mock_store_open, mock_rekey):
        config = {
            "key_derivation_method": self.key_derivation_method,
            "key": self.key,
            "storage_type": self.storage_type,
            "rekey": "rekey",
        }

        mock_store_open.side_effect = [
            AskarError(AskarErrorCode.ENCRYPTION, message="testing"),
            mock.AsyncMock(auto_spec=True),
        ]

        store = await AskarStoreConfig(config).open_store()

        assert isinstance(store, AskarOpenStore)
        assert mock_rekey.called

    @mock.patch.object(Store, "open")
    @mock.patch.object(Store, "rekey")
    async def test_open_store_fail_retry_with_rekey_fails(
        self, mock_store_open, mock_rekey
    ):
        config = {
            "key_derivation_method": self.key_derivation_method,
            "key": self.key,
            "storage_type": self.storage_type,
            "rekey": "rekey",
        }

        mock_store_open.side_effect = [
            AskarError(AskarErrorCode.ENCRYPTION, message="testing"),
            mock.AsyncMock(auto_spec=True),
        ]

        store = await AskarStoreConfig(config).open_store()

        assert isinstance(store, AskarOpenStore)
        assert mock_rekey.called

    def test_unsupported_storage_type(self):
        with self.assertRaises(ProfileError) as ctx:
            AskarStoreConfig({"storage_type": "invalid"})
        assert "Unsupported storage type" in str(ctx.exception)

    def test_get_uri_sqlite_memory(self):
        config = {
            "storage_type": "sqlite",
            "name": "test",
        }
        askar_store = AskarStoreConfig(config)
        uri = askar_store.get_uri(in_memory=True)
        assert uri == "sqlite://:memory:"

    def test_get_uri_postgres(self):
        config = {
            "storage_type": "postgres",
            "name": "testname",
            "storage_config": json.dumps({"url": "localhost", "connection_timeout": 5}),
            "storage_creds": json.dumps({"account": "user", "password": "pass"}),
        }
        askar_store = AskarStoreConfig(config)
        uri = askar_store.get_uri()
        assert uri.startswith("postgres://user:pass@localhost/testname")

    def test_postgres_config_missing_fields(self):
        config = {
            "storage_type": "postgres",
            "storage_config": json.dumps({}),  # missing url
            "storage_creds": json.dumps({"account": "user", "password": "pass"}),
        }

        with self.assertRaises(ProfileError) as ctx:
            AskarStoreConfig(config)._validate_postgres_config()
        assert "Missing 'url'" in str(ctx.exception)

    @mock.patch(
        "aries_askar.Store.remove",
        side_effect=AskarError(AskarErrorCode.NOT_FOUND, message="Store not found"),
    )
    async def test_remove_store_not_found(self, _):
        config = {"storage_type": "sqlite", "name": "nonexistent"}
        store_config = AskarStoreConfig(config)
        with self.assertRaises(ProfileNotFoundError):
            await store_config.remove_store()

    @mock.patch(
        "aries_askar.Store.remove",
        side_effect=AskarError(AskarErrorCode.UNEXPECTED, message="Some error"),
    )
    async def test_remove_store_other_error(self, _):
        config = {"storage_type": "sqlite", "name": "badstore"}
        store_config = AskarStoreConfig(config)
        with self.assertRaises(ProfileError):
            await store_config.remove_store()

    def test_askar_open_store_name_property(self):
        config = AskarStoreConfig({"storage_type": "sqlite", "name": "teststore"})
        store = mock.AsyncMock()
        open_store = AskarOpenStore(config=config, created=True, store=store)
        assert open_store.name == "teststore"

    async def test_askar_open_store_close(self):
        config = AskarStoreConfig({"storage_type": "sqlite", "auto_remove": True})
        store = mock.AsyncMock()
        open_store = AskarOpenStore(config=config, created=True, store=store)

        await open_store.close()

        store.close.assert_awaited_with(remove=True)
        assert open_store.store is None
